--------------------------------------------------------------------------
-- GTFO_Skada.lua 
--------------------------------------------------------------------------
--[[
GTFO & Skada Integration
Author: Zensunim of Malygos

Change Log:
	v4.6
		- Added Skada Integration

]]--

function GTFO_Skada()
	local L = LibStub("AceLocale-3.0"):GetLocale("Skada");
	local Skada = Skada
	local mod = Skada:NewModule("GTFO");
	local playermod = Skada:NewModule("GTFO alert list");
	local alertmod = Skada:NewModule("GTFO spell list");
	
	local function log_alert(set, alert)
		-- Get the player.
		local player = Skada:get_player(set, alert.playerID, alert.playerName);
		if player then
			set.alerts = set.alerts + 1;
			player.alertCount = player.alertCount + 1;
			
			if not player.alert[alert.alertID] then
				player.alert[alert.alertID] = {
					name = GTFO_GetAlertType(alert.alertID),
					alerts = 0,
					damage = 0,
					spell = { }
				};
			end
			
			player.alert[alert.alertID].alerts = player.alert[alert.alertID].alerts + 1;
			
			-- Also add to set total damage.
			set.damage = set.damage + alert.damage;
			player.alertDamage = player.alertDamage + alert.damage;
			
			-- Add spell to player if it does not exist.
			if not player.alert[alert.alertID].spell[alert.spellName] then
				player.alert[alert.alertID].spell[alert.spellName] = {
						id = alert.spellID, 
						alertID = alert.alertID,
						count = 0, 
						damage = 0, 
				};
			end
    		
			-- Get the spell from player.
			local spell = player.alert[alert.alertID].spell[alert.spellName];
			spell.count = spell.count + 1;
			spell.damage = spell.damage + alert.damage;
			if spell.max == nil or alert.damage > spell.max then
				spell.max = alert.damage;
			end
			if (spell.min == nil or alert.damage < spell.min) then
				spell.min = amount;
			end
		end
	end
	
	local alert = {};

	-- Alert overview.
	function mod:Update(win, set)
		-- Max value.
		local max = 0
		
		local nr = 1
		for i, player in ipairs(set.players) do
			if player.alerts > 0 then
				local d = win.dataset[nr] or {}
				win.dataset[nr] = d
				d.label = player.name
				d.valuetext = Skada:FormatValueText(
												Skada:FormatNumber(player.alertDamage), self.metadata.columns.Damage, 
												player.alerts, self.metadata.columns.DPS
											)
				d.value = player.alertDamage
				d.id = player.id
				d.class = player.class
				if player.alertDamage > max then
					max = player.alertDamage
				end
				nr = nr + 1
			end
		end
		
		win.metadata.maxvalue = max
	end
	
	-- Tooltip for a specific player.
	local function player_tooltip(win, id, label, tooltip)
		local player = Skada:find_player(win:get_selected_set(), playermod.playerid)
		if player then
			local spell = player.damagespells[label]
			if spell then
				tooltip:AddLine(player.name.." - "..label)
				if spell.max and spell.min then
					tooltip:AddDoubleLine(L["Minimum hit:"], Skada:FormatNumber(spell.min), 255,255,255,255,255,255)
					tooltip:AddDoubleLine(L["Maximum hit:"], Skada:FormatNumber(spell.max), 255,255,255,255,255,255)
				end
				tooltip:AddDoubleLine(L["Average hit:"], Skada:FormatNumber(spell.damage / spell.totalhits), 255,255,255,255,255,255)
			end
		end
	end
	
	function playermod:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), id)
		playermod.playerid = id
		playermod.title = player.name..L["'s Damage"]
	end
	
	-- Detail view of a player.
	function playermod:Update(win, set)
		-- View spells for this player.
			
		local player = Skada:find_player(set, self.playerid)
		local max = 0
		
		-- If we reset we have no data.
		if player then
			
			local nr = 1
			if player then
				for alertID, alert in pairs(player.alert) do
	
					local d = win.dataset[nr] or {}
					win.dataset[nr] = d
					d.label = alert.name
					d.id = alertID
					--d.icon = select(3, GetSpellInfo(spell.id))
					d.value = alert.alerts
					d.valuetext = Skada:FormatValueText(
													Skada:FormatNumber(alert.damage), self.metadata.columns.Damage,
													alert.alerts, self.metadata.columns.DPS,
													string.format("%02.1f%%", alert.damage / player.alertDamage * 100), self.metadata.columns.Percent
												)
					if alert.damage > max then
						max = alert.damage
					end
					nr = nr + 1
				end
			end
		end
		
		win.metadata.maxvalue = max
	end
	
	
	function alertmod:Enter(win, id, label)
		local player = Skada:find_player(win:get_selected_set(), playermod.playerid)
		spellmod.spellname = label
		spellmod.title = player.name..L["'s "]..label
	end
	
	function alertmod:Update(win, set)
		local player = Skada:find_player(set,playermod.playerid)
		
		if player then
			local spell = player.damagespells[self.spellname]
			
			if not player.alert[alert.alertID].spell[self.spellName] then
				if spell then
					win.metadata.maxvalue = spell.totalhits
					
				end
			end
		end
	end
	
	function mod:OnEnable()
		mod.metadata = {
			showspots = true, 
			click1 = playermod,
			columns = {Damage = true, DPS = true, Percent = false}
		};
		playermod.metadata = {
			tooltip = player_tooltip, 
			click1 = alertmod, 
			columns = {Damage = true, DPS = true, Percent = true}
		};
		alertmod.metadata = {
			columns = {Damage = true, Percent = true}
		};
		
		Skada:AddMode(self)
	end
	
	function mod:OnDisable()
		Skada:RemoveMode(self)
	end
	
	function mod:AddToTooltip(set, tooltip)
	 	GameTooltip:AddDoubleLine(L["DPS"], ("%02.1f"):format(getRaidDPS(set)), 1,1,1)
	end
	
	function mod:GetSetSummary(set)
		return Skada:FormatNumber(set.damage)
	end
	
	-- Called by Skada when a new player is added to a set.
	function mod:AddPlayerAttributes(player)
		if not player.damage then
			player.damage = 0
			player.damagespells = {}
		end
		if not player.damaged then
			player.damaged = {}
		end
	end
	
	-- Called by Skada when a new set is created.
	function mod:AddSetAttributes(set)
		if not set.damage then
			set.damage = 0
		end
	end

	function GTFO_RecordSkada(sourceName, sourceID, alertID, spellID, spellName, damage)
	  local alert = {
	  	alertID = alertID,
	  	damage = damage,
	  	spellID = spellID,
	  	spellName = spellName,
	  	playerID = sourceID,
	  	playerName = sourceName
	  };
	
		log_alert(Skada.current, alert);
		log_alert(Skada.total, alert);
	end

	GTFO_DebugPrint("Skada integration loaded.");
	
	return true;	
end

